#include <SoftwareSerial.h>



union laser_send_data_t {
  struct bytes_t {
    uint8_t preamble;
    uint8_t command;
    uint8_t data[3];
    uint8_t check_code[2];
  } bytes;
  uint8_t array[sizeof(bytes)];
} laser_send_data;

union laser_rcv_data_t {
  struct bytes_t {
    uint8_t preamble;
    uint8_t command;
    uint8_t result;
    uint8_t data[3];
    uint8_t check_code[2];
  } bytes;
  uint8_t array[sizeof(bytes)];
} laser_rcv_data;

int16_t laser_distance = 0;


void setup()
{
  pinMode(22, OUTPUT);
  pinMode(23, OUTPUT);
  digitalWrite(22, LOW);
  digitalWrite(23, LOW);
  digitalWrite(22, HIGH);
  digitalWrite(23, HIGH);
  delay(50);
  digitalWrite(23, LOW);
  digitalWrite(23, LOW);

  laser_send_data.bytes.preamble = 0xAA;
  laser_send_data.bytes.command = 0x03;
  // Open serial communications and wait for port to open:
  Serial.begin(115200);

  // set the data rate for the SoftwareSerial port
  Serial2.begin(38400);
}

void loop() // run over and over
{
  laser_send_data.bytes.check_code[0] = (uint8_t)(laser_send_data.bytes.preamble + laser_send_data.bytes.command + laser_send_data.bytes.data[0] + laser_send_data.bytes.data[1] + laser_send_data.bytes.data[2]) << 8;
  laser_send_data.bytes.check_code[1] = (uint8_t)(laser_send_data.bytes.preamble + laser_send_data.bytes.command + laser_send_data.bytes.data[0] + laser_send_data.bytes.data[1] + laser_send_data.bytes.data[2]);


  Serial2.write(laser_send_data.array, sizeof(laser_send_data.array));
  uint16_t checksum = 0;
  if (Serial2.available()) {
    Serial2.readBytes(laser_rcv_data.array, sizeof(laser_rcv_data.array));
    for (uint8_t i = 0; i < sizeof(laser_rcv_data.array) - 2; i++) {
      checksum = checksum + laser_rcv_data.array[i];
    }

    /*
    Serial.println(checksum, HEX);
    Serial.println(laser_rcv_data.bytes.check_code[0], HEX);
    Serial.println(laser_rcv_data.bytes.check_code[1], HEX);
    */

    if ((laser_rcv_data.bytes.command == 0x03) && (laser_rcv_data.bytes.result == 0x06) && (checksum == (laser_rcv_data.bytes.check_code[0] << 8 | laser_rcv_data.bytes.check_code[1]))) {
      laser_distance = (int16_t)((laser_rcv_data.bytes.data[0] << 16) | (laser_rcv_data.bytes.data[1] << 8) | laser_rcv_data.bytes.data[2]);
    }
    else {
      laser_distance = -1;
    }
  }
  /*
  Serial.println("Request");
  Serial.write(laser_send_data.array, sizeof(laser_send_data.array));
  Serial.println("");
  Serial.println("Answer");
  Serial.write(laser_rcv_data.array, sizeof(laser_rcv_data.array));
  Serial.println("");
  */
  Serial.print("Distance is ");
  Serial.print (laser_distance);
  Serial.println(" mm");

  delay(10000);
}

